//-----------------------------------------------------------------------------
// File: PixelMotionBlur.fx
//
// Desc: Effect file for image based motion blur. The HLSL shaders are used to
//       calculate the velocity of each pixel based on the last frame's matrix 
//       transforms.  This per-pixel velocity is then used in a blur filter to 
//       create the motion blur effect.
// 
// Copyright (c) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
float4 MaterialAmbientColor;
float4 MaterialDiffuseColor;

float3 LightDir = normalize(float3(1.0f, 1.0f, 1.0f));
float4 LightAmbient = { 1.0f, 1.0f, 1.0f, 1.0f };    // ambient
float4 LightDiffuse = { 1.0f, 1.0f, 1.0f, 1.0f };    // diffuse

float DistortionPosx ;//= (float2(0.0f, 0.0f));
float DistortionPosy ;

texture RenderTargetTexture;
texture RenderTargetTexture2;
texture MeshTexture;
texture CurFrameVelocityTexture;
texture LastFrameVelocityTexture;

float4x4 mWorld;
float4x4 mWorldViewProjection;
float4x4 mWorldViewProjectionLast;

float PixelBlurConst = 1.0f;
static const float NumberOfPostProcessSamples = 12.0f;
float ConvertToNonHomogeneous;
float VelocityCapSq = 1.0f;
float RenderTargetWidth;
float RenderTargetHeight;

float MagicValue;;

float2 MagicArray[100];

float vertexin;
//float MagicArray2[10000];
//-----------------------------------------------------------------------------
// Texture samplers
//-----------------------------------------------------------------------------
sampler RenderTargetSampler = 
sampler_state
{
    Texture = <RenderTargetTexture>;
    MinFilter = POINT;  
    MagFilter = LINEAR;

    AddressU = Clamp;
    AddressV = Clamp;
};
sampler RenderTargetSampler2 = 
sampler_state
{
    Texture = <RenderTargetTexture2>;
    MinFilter = POINT;  
    MagFilter = LINEAR;

    AddressU = Clamp;
    AddressV = Clamp;
};
sampler CurFramePixelVelSampler = 
sampler_state
{
    Texture = <CurFrameVelocityTexture>;
    MinFilter = POINT;
    MagFilter = POINT;

    AddressU = Clamp;
    AddressV = Clamp;
};

sampler LastFramePixelVelSampler = 
sampler_state
{
    Texture = <LastFrameVelocityTexture>;
    MinFilter = POINT;
    MagFilter = POINT;

    AddressU = Clamp;
    AddressV = Clamp;
};

sampler MeshTextureSampler = 
sampler_state
{
    Texture = <MeshTexture>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};


//-----------------------------------------------------------------------------
// Vertex shader output structure
//-----------------------------------------------------------------------------
struct VS_OUTPUT
{
    float4 Position : POSITION;   // position of the vertex
    float4 Diffuse  : COLOR0;     // diffuse color of the vertex
    float2 TextureUV : TEXCOORD0;  // typical texture coords stored here
   // float2 VelocityUV : TEXCOORD1;  // per-vertex velocity stored here
};


//-----------------------------------------------------------------------------
// Name: WorldVertexShader     
// Type: Vertex shader                                      
// Desc: In addition to standard transform and lighting, it calculates the velocity 
//       of the vertex and outputs this as a texture coord.
//-----------------------------------------------------------------------------
VS_OUTPUT WorldVertexShader( float3 vPos : POSITION, 
                             float4 Diffuse  : COLOR0,
                             float2 vTexCoord0 : TEXCOORD0 )
{
    VS_OUTPUT Output;


    // Output the vetrex position in projection space
    Output.Position = mul(vPos,mWorldViewProjection);


//vertexin=1.0f;//vPos.x;
//Output.Position.z=1.0f;
//Output.Position=1.0f;
    // Store the velocity in a texture coord
    //Output.VelocityUV = 1.0f;
        
    // Compute simple lighting equation
    Output.Diffuse.rgb = 1.0f;                
    Output.Diffuse.a = 1.0f;   
    // Just copy the texture coordinate through
    Output.TextureUV = vTexCoord0; 
    
    return Output;    
}


//-----------------------------------------------------------------------------
// Pixel shader output structure
//-----------------------------------------------------------------------------
struct PS_OUTPUT
{
    // The pixel shader can output 2+ values simulatanously if 
    // d3dcaps.NumSimultaneousRTs > 1
    
    float4 RGBColor      : COLOR0;  // Pixel color    
    float4 PixelVelocity : COLOR1;  // Pixel velocity 
};


//-----------------------------------------------------------------------------
// Name: WorldPixelShader                                        
// Type: Pixel shader
// Desc: Uses multiple render targets (MRT) to output 2 values at once from a 
//       pixel shader.  This shader outputs the pixel's color and velocity.
//-----------------------------------------------------------------------------
PS_OUTPUT WorldPixelShader( VS_OUTPUT In )
{ 
    PS_OUTPUT Output;

    // Lookup mesh texture and modulate it with diffuse
    Output.RGBColor = tex2D(MeshTextureSampler, In.TextureUV) * In.Diffuse;

    // Velocity of this pixel simply equals the linear interpolation of the 
    // velocity of this triangle's vertices.  The interpolation is done
    // automatically since the velocity is stored in a texture coord.
    // We're storing the velocity in .R & .G channels and the app creates 
    // a D3DFMT_G16R16F texture to store this info in high precison.
    //Output.PixelVelocity = float4(In.VelocityUV,1.0f,1.0f);
    
    // Using MRT, output 2 values in the pixel shader.    
    return Output;
}



float4 PostProcessAlphaToRGBPS( float2 OriginalUV : TEXCOORD0 ) : COLOR
{
	OriginalUV.y=0.0f-OriginalUV.y;
	 OriginalUV=OriginalUV*0.5;
                OriginalUV.x+=0.50f;
       OriginalUV.y+=0.50f;
       
       
       
	float4 Current = tex2D(RenderTargetSampler, OriginalUV);

	return Current;
}
//-----------------------------------------------------------------------------
// Name: PostProcessMotionBlurPS 
// Type: Pixel shader                                      
float4 PostProcessBlooomPS( float2 OriginalUV : TEXCOORD0 ) : COLOR
{
	OriginalUV.y=0.0f-OriginalUV.y;
	 OriginalUV=OriginalUV*0.5;
                OriginalUV.x+=0.50f;
       OriginalUV.y+=0.50f;
       
       
     float4 Other  = tex2D(RenderTargetSampler2, OriginalUV);
       
	float4 Current = tex2D(RenderTargetSampler, OriginalUV);
	
	float4 Test;
	float2 NewUV;
	for (int numx=0;numx<3;numx++)
    {
		for (int numy=0;numy<3;numy++)
		{
			NewUV=OriginalUV;
			NewUV.x+=0.015625*(numx-1);
			NewUV.y+=0.015625*(numy-1);
			
			Test=tex2D(RenderTargetSampler, NewUV);
			Test*=1.0f/(1+abs(numy-1));
			Current=max(Current,Test);
		}	        
    }
		
	//Current-=0.50f;
	Current*=0.0f;
	if (Current.x<0.0f)	
	{
		Current.x=0.0f;
	}
	if (Current.y<0.0f)	
	{
		Current.y=0.0f;
	}
	if (Current.z<0.0f)	
	{
		Current.z=0.0f;
	}
	Current=max(Current,Other);
	return Current;
}
float4 PostProcessMotionBlurPS( float2 OriginalUV : TEXCOORD0 ) : COLOR
{
    float2 pixelVelocity;
    OriginalUV.y=0.0f-OriginalUV.y;
    // Get this pixel's current velocity and this pixel's last frame velocity
    // The velocity is stored in .r & .g channels
    //float4 curFramePixelVelocity = tex2D(CurFramePixelVelSampler, OriginalUV);
    //float4 lastFramePixelVelocity = tex2D(LastFramePixelVelSampler, OriginalUV);

    // For each sample, sum up each sample's color in "Blurred" and then divide
    // to average the color after all the samples are added.
    
    //OriginalUV.x=OriginalUV.x*0.5f; 
    //for(float i = 0; i < NumberOfPostProcessSamples; i++)
    
   
    //{   
        // Sample texture in a new spot based on pixelVelocity vector 
        // and average it with the other samples        
        //float2 lookup = pixelVelocity * i / NumberOfPostProcessSamples + OriginalUV;
        
        // Lookup the color at this new spot
        //if (OriginalUV.x<0.0f)OriginalUV.x=0.0f;
        float2 Dist=OriginalUV;
        float2 DistortionPos;
        DistortionPos.x=DistortionPosx;
        DistortionPos.y=DistortionPosy;
        
        //DistortionPos.x=MagicArray[0].x;
       // DistortionPos.y=MagicArray[0].y;
        
        Dist=Dist-DistortionPos;
        float len=length(Dist);
        
       // if (len<0.1f)len=0.1f;//MagicValue
		if (len<MagicValue)
		{
			float otherMagic=len;
			float themagic=1.0f+2*(MagicValue-otherMagic)*(MagicValue-otherMagic);
		
		//do distortion stuff
			
			//OriginalUV=DistortionPos+((OriginalUV-DistortionPos)*themagic);
        }

      //OriginalUV=-OriginalUV;
//OriginalUV.y=-OriginalUV.y;
        OriginalUV=OriginalUV*0.5;
                OriginalUV.x+=0.50f;
       OriginalUV.y+=0.50f;
        
  
        //OriginalUV.y=1.0f-OriginalUV.y;
        float4 Current = tex2D(RenderTargetSampler, OriginalUV);
		float4 Test;
        float3 Bloomed=Current.rgb;
        float2 NewUV=OriginalUV;
        float diff=0;
        float diff2=0;
        
        for (int numx=0;numx<3;numx++)
        {
			for (int numy=0;numy<3;numy++)
			{
				NewUV=OriginalUV;
				NewUV.x+=0.001f*(numx-1);
				NewUV.y+=0.001f*(numy-1);
				Test=tex2D(RenderTargetSampler, NewUV);
				diff=abs(Bloomed-Test);
				diff2=diff2+diff;
				//Bloomed=max(Bloomed,Test);
			}	        
        }
        if (diff2>0.00)
        {
        Bloomed.x+=diff2;
        Bloomed.y+=diff2;
        Bloomed.z+=diff2;
        }
 //       if (Bloomed.x>0.25 && Bloomed.y>0.25 && Bloomed.z>0.25)
 //       {
//			 Bloomed.x*=2;
//			 Bloomed.y*=2;
//			 Bloomed.z*=2;
 //       }
        //else
        //{
		//	Bloomed.x/=2;
		//	Bloomed.y/=2;
		//	Bloomed.z/=2;
        //}
        
        // Add it with the other samples
        Current.rgb=Bloomed;//1.0f-Current.rgb;
        
        
        //Current.rgb=0.0f;
        //Current=vertexin;
        
       
        //Current.rgb=len;
       // Current.rgb.z=len;//*MagicValue;
        //Current.x+=length(Current.rgb);
   // }
   //if (Current.rgb.x>0.50f)
   //{
	//	Current.rgb.x=0.0f;
//   }
    
    // Return the average color of all the samples
    //return float4(1.0f,1.0f,1.0f,1.0f);//Blurred / NumberOfPostProcessSamples, 1.0f);
    return Current;//float4(Blurred , 1.0f);
}


//-----------------------------------------------------------------------------
// Name: WorldWithVelocityMRT
// Type: Technique                                     
// Desc: Renders the scene's color to render target 0 and simultaneously writes 
//       pixel velocity to render target 1.
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Name: PostProcessMotionBlur
// Type: Technique                                     
// Desc: Renders a full screen quad and uses velocity information stored in 
//       the textures to blur image.
//-----------------------------------------------------------------------------
technique PostProcessMotionBlur
{
    pass P0
    {  
		//VertexShader = compile vs_2_0 WorldVertexShader();      
        PixelShader = compile ps_2_0 PostProcessMotionBlurPS();
    }
}
technique PostProcessAlphaToRGB
{
    pass P0
    {       
		//VertexShader = compile vs_2_0 WorldVertexShader();       
        PixelShader = compile ps_2_0 PostProcessAlphaToRGBPS();
    }
}
technique PostProcessBloom
{
    pass P0
    {       		
        PixelShader = compile ps_2_0 PostProcessBlooomPS();
    }
}


